/*
   This program parses and then evaluates strings from Language_3.
*/

public class Language_3_Examples_1
{
   public static void main(String[] args)
   {
      String[] programs =
      {
         "( ( lambda ( x ) ( * x x ) ) 7 )", // apply an anonymous lambda expression

         // This next example shows that var can return a function
         // value which can be directly applied to an input.
         // Notice that g does not play much of any role in this
         "( var r ( ( var g ( lambda ( x ) ( * x x ) ) ) 5 ) )",

         "( prog " +
           "( var g ( lambda ( h ) ( h 5 ) ) ) " +
           "( g ( lambda ( x ) ( * x x ) ) ) )", // pass an anonymous lambda as a parameter

         // apply an anonymous function (with a function parameter) to an anonymous function
         "( ( lambda ( h ) ( h 7 ) ) ( lambda ( x ) ( + x x x ) ) )",

         // anonymous functions as return values
         "( prog " +
           "( var h ( lambda ( b ) ( if b " +
                                     "( lambda ( x ) ( * x x ) ) " +         // anonymous lambda
                                     "( lambda ( x ) ( * x x x ) ) ) ) ) " + // anonymous lambda
           "( var r1 ( ( h  true ) 5 ) ) " +  // returns 25
           "( print r1 ) " +
           "( var r2 ( ( h false ) 5 ) ) ) ", // returns 125

         // another anonymous function as a return value.
         // (With dynamic scope, this program would return a different result.)
         "( prog " +
           "( var x -100 ) " +
           "( var f ( lambda ( x ) ( lambda ( y ) ( + y x ) ) ) ) " +
           "( ( f 5 ) 100 ) )",

         // Demonstrate a nested anonymous function parameter.
         // The final result should be 321.
         // (It would be 221 using dynamic scope.)
         "( prog " +
           "( var w 300 ) " +                  // w is a global variable
           "( var f1 ( lambda ( x ) ( f2 ( lambda ( y ) ( + x y w ) ) ) ) ) " +
           "( var f2 ( lambda ( z ) " +        // z is a function parameter
                          "( var w 200 ) " +   // w is a local variable
                          "( z ( / w 10 ) ) ) ) " + // call the function parameter
           "( f1 1 ) )"
      };

      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =============================================");

         try // to parse the program.
         {
            // Build the abstract syntax tree that represents the program.
            Tree ast = Parse.parse( programs[i] );

            // Pretty-print the abstract syntax tree.
            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

            // Print the infix version of the program.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            try // to evaluate the program (interpret the AST).
            {
               Value value = Evaluate.eval( ast );
               System.out.println( "result = " + value + "\n" );
            }
            catch (EvalException e)
            {
             //System.out.println( e );
               e.printStackTrace();
            }

            if(true)
            try // to create dot and png files from the AST.
            {
               // Create an empty dot file.
               String baseName = String.format("Language_3_Examples_1.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
             //e.printStackTrace();
            }
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()

}//Language_3_Examples_1
