/*
   This program parses and then evaluates strings from Language_2.

   These examples demonstrate two ways that a function can be
   used as a first class value.
   (1) A function can be passed as a parameter to another function.
   (2) A function can be returned by another function
*/

public class Language_3_Examples_2
{
   public static void main(String[] args)
   {
      String[] programs =
      {
         "( prog " +  // mutually recursive functions
           "( fun even ( lambda ( n ) " +
                         "( if ( == n 0 ) " +
                            "true " +
                            "( if ( == n 1 ) " +
                               "false " +
                               "( odd ( - n 1 ) ) ) ) ) ) " +
           "( fun odd  ( lambda ( n ) " +
                         "( if ( == n 0 ) " +
                            "false " +
                            "( if ( == n 1 ) " +
                               "true " +
                               "( even ( - n 1 ) ) ) ) ) ) " +
           "( even 5 ) )"
      };

      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =============================================");

         try // to parse the program.
         {
            // Build the abstract syntax tree that represents the program.
            Tree ast = Parse.parse( programs[i] );

            // Pretty-print the abstract syntax tree.
            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

            // Print the infix version of the program.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            try // to evaluate the program (interpret the AST).
            {
               Value value = Evaluate.eval( ast );
               System.out.println( "result = " + value + "\n" );
            }
            catch (EvalException e)
            {
             //System.out.println( e );
               e.printStackTrace();
            }

            if(true)
            try // to create dot and png files from the AST.
            {
               // Create an empty dot file.
               String baseName = String.format("Language_3_Examples_2.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
             //e.printStackTrace();
            }
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()

}//Language_3_Examples_2
