/*
   This program parses and then evaluates strings from Language_3.
*/

public class Language_3_Examples_4
{
   public static void main(String[] args)
   {
      String[] programs =
      {
/*
         // With static scope, this example would be an infinite loop.
         // With dynamic scope, this example terminates.
         "( prog " +
           "( var f1 ( lambda ( x ) ( f2 0 ) ) ) " +  // global f1
           "( var f2 ( lambda ( y ) " +
                           "( var f1 ( lambda ( w ) 10 ) ) " +  // nested f1
                           "( f3 0 ) ) ) " +
           "( var f3 ( lambda ( z ) ( f1 0 ) ) ) " +   // which f1 is called?
           "( f1 0 ) )",
*/
/*
         // Another (convoluted) example demonstrating dynamic scope.
         // (Notice that, with static scope, h is undefined in f, and x is undefined in h.)
         "( prog " +
           "( var f ( lambda ( x ) ( h x ) ) ) " +  // non local reference to h
           "( var g ( lambda ( y ) " +
                         "( var h ( lambda ( z ) ( + z x ) ) ) " + // non local ref to x
                         "( f y ) ) ) " +
           "( g 5 ) )",
*/
         // The final result should be 100.
         // (It would be 10 using dynamic scope.)
         "( prog " +
           "( var f1 ( lambda ( x ) 100 ) ) " +  // global f1
           "( var f2 ( lambda ( y ) " +
                           "( var f1 ( lambda ( w ) 10 ) ) " +  // nested f1
                           "( f3 0 ) ) ) " +
           "( var f3 ( lambda ( z ) ( f1 0 ) ) ) " +   // which f1 is called?
           "( f2 0 ) )",

         // Demonstrate a function parameter.
         // The final result should be 321.
         // (It would be 421 using dynamic scope.)
         "( prog " +
           "( var w 300 ) " +
           "( fun g1 ( lambda ( x ) " +
                         "( var g2 ( lambda ( y ) ( + ( + x y ) w ) ) ) " + // nested function
                         "( g3 g2 ) ) ) " + // nested function passed as a parameter
           "( fun g3 ( lambda ( z ) " +     // z is a function parameter
                         "( var w 400 ) " +
                         "( z 20 ) ) ) " + // call the function parameter
           "( g1 1 ) )",

         // Show that a function can be passed as a parameter.
         // The final value should be 10.
         // (With dynamic scope, the final result would be different.)
         "( prog " +
           "( var n 2 ) " +
           "( var f ( lambda ( x ) ( * n x ) ) ) " +
           "( var g ( lambda ( h ) ( h 5 ) ) ) " +
           "( set n 10 ) " +
           "( g f ) )",    // pass f as an argument to g

         // Show that a function can be the return value.
         // (With dynamic scope, this program would return a different result.)
         "( prog " +
           "( var x -100 ) " +
           "( var f ( lambda ( x ) " +
                        "( var g ( lambda ( y ) ( + y x ) ) ) " +
                        "g ) ) " +  // f returns g
           "( ( f 5 ) 100 ) )"
      };

      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =============================================");

         try // to parse the program.
         {
            // Build the abstract syntax tree that represents the program.
            Tree ast = Parse.parse( programs[i] );

            // Pretty-print the abstract syntax tree.
            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

            // Print the infix version of the program.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            try // to evaluate the program (interpret the AST).
            {
               Value value = Evaluate.eval( ast );
               System.out.println( "result = " + value + "\n" );
            }
            catch (EvalException e)
            {
             //System.out.println( e );
               e.printStackTrace();
            }

            if(true)
            try // to create dot and png files from the AST.
            {
               // Create an empty dot file.
               String baseName = String.format("Language_3_Examples_4.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
             //e.printStackTrace();
            }
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()

}//Language_3_Examples_3
